// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "display/RenderTarget.hpp"

namespace Dawn {
  enum CameraType {
    CAMERA_TYPE_ORTHONOGRAPHIC,
    CAMERA_TYPE_PERSPECTIVE
  };

  class Camera : public SceneItemComponent {
    public:
      glm::mat4 projection;
      std::shared_ptr<RenderTarget> target = nullptr;

      // Perspective
      enum CameraType type = CAMERA_TYPE_PERSPECTIVE;
      float_t fov = 0.785398f;// 45 degrees

      // Ortho
      float_t orthoLeft = 0.0f;
      float_t orthoRight = 1.0f;
      float_t orthoBottom = 0.0f;
      float_t orthoTop = 1.0f;

      // Shared
      float_t clipNear = 0.001f;
      float_t clipFar = 100.0f;

      /**
       * Create a new Camera Component.
       * 
       * @param item SceneItem that this component belongs to.
       */
      Camera(SceneItem &item);

      /**
       * Updates the projection matrix.
       */
      void updateProjection();

      void lookAt(glm::vec3 position, glm::vec3 look);
      void lookAt(glm::vec3 position, glm::vec3 look, glm::vec3 up);

      /**
       * Returns the intended render target for this camera to render to, will
       * automatically revert to the back buffer if no frame buffer is provided.
       * 
       * @return The target render target framebuffer.
       */
      RenderTarget & getRenderTarget();

      /**
       * Returs the aspect ratio of the camera.
       * 
       * @return The aspect ratio of the camera.
       */
      float_t getAspect();

      /**
       * Event triggered by the scene item when the item is added to the scene.
       */
      void start() override;
  };
}